% Function implementing the attitude dynamics of a floating platform
% embeded with an algorithm that estimates the position of the center of
% mass of the system for automatic balancing purposes.
%
% =========================================================================
%
% INPUT VARIABLES:
%
% RG_SYS : position of the center of mass of the floating system with
%          respect to the body frame with origin on the center of rotation
%          of the system. Unit [m].
% noiseIMU : noise of the accelerometer. The noise is assumed to
%            be gaussian with zero mean. Unit [m/s^2].
% noiseGYRO : noise of the mems gyro. The noise is assumed to be
%            gaussian with zero mean. Unit [rad/s^2].
% visFlag : (optional) parameter activating the amination of the S/C motion
%
%
% OUTPUT VARIABLE:
% 
% rg_sys: estimated RG_SYS. Unit [m].
%
% =========================================================================
%
% Example:
% r = balancing_model([.002 -.002, -0.005]',1,2,1);
%
% =========================================================================
%
% Author        : Alberto Gallina
% Last Update   : 13.02.2019
%
% =========================================================================

function rg_sys = balancing_model(RG_SYS, noiseIMU, noiseGyro, visFlag)

 
%% user defined parameters
% rotation angles for inertia tensor [rad]
pitchJ = 0*pi/180;
rollJ  = 0*pi/180;

% moments and products of inertia calculated with respect to the body frame with origin
% at the center of rotation of the body [kgm2]
Jx  = 4e-4;
Jy  = 4e-4;
Jz  = 8e-4;
Jxy = 0.0;
Jxz = 0.0;
Jyz = 0.0;

% G of the S/C wrt the body frame[m]
% RG_SC = 0.001*[-1 1 -3]';
RG_SC = RG_SYS;

% Mass of the S/C in [kg]
M_SC = 1.1;


% INITIAL CONDITIONS

% initial orientation [rad]
yaw0    = 0*pi/180;      
pitch0  = 0*pi/180;      
roll0   = 0*pi/180;      

% initial angular velocity [rad/s]
wx0 = 0.0;      
wy0 = 0.0;      
wz0 = 0.0;      

% sensor calibration parameters
GyroGainX  = 1;
GyroGainY  = 1;
GyroGainZ  = 1;
GyroXtalk13 = 90*pi/180;
GyroXtalk32 = 90*pi/180;
GyroXtalk12 = 90*pi/180;
GyroOffsetX = 0;
GyroOffsetY = 0;
GyroOffsetZ = 0;
GyroNoise = noiseGyro;

IMUGainX  = 1;
IMUGainY  = 1;
IMUGainZ  = 1;
IMUXtalk13 = 90*pi/180;
IMUXtalk32 = 90*pi/180;
IMUXtalk12 = 90*pi/180;
IMUOffsetX = 0;
IMUOffsetY = 0;
IMUOffsetZ = 0;
IMUNoise = noiseIMU;

%% calculate inertia tensor
RJ = [ cos(pitchJ)  sin(pitchJ)*sin(rollJ)  sin(pitchJ)*cos(rollJ); ...
       0            cos(rollJ)             -sin(rollJ); ...
      -sin(pitchJ)  cos(pitchJ)*sin(rollJ)  cos(pitchJ)*cos(rollJ) ];


J0 = [Jx -Jxy -Jxz; -Jxy Jy -Jyz; -Jxz -Jyz Jz];
J = RJ*J0*RJ';

%% create structure variables for model parameters
initialConds.orient = [yaw0, pitch0, roll0];
initialConds.omega  = [wx0, wy0, wz0];

paramSC.RG = RG_SC;
paramSC.J  = J;
paramSC.Mass = M_SC;

paramSensor.Gyro.a11 = GyroGainX;
paramSensor.Gyro.a22 = GyroGainY;
paramSensor.Gyro.a33 = GyroGainZ;
paramSensor.Gyro.a13 = GyroXtalk13;
paramSensor.Gyro.a32 = GyroXtalk32;
paramSensor.Gyro.a12 = GyroXtalk12;
paramSensor.Gyro.a1  = GyroOffsetX;
paramSensor.Gyro.a2  = GyroOffsetY;
paramSensor.Gyro.a3  = GyroOffsetZ;
paramSensor.Gyro.sigmaNoise = GyroNoise;

paramSensor.IMU.a11 = IMUGainX;
paramSensor.IMU.a22 = IMUGainY;
paramSensor.IMU.a33 = IMUGainZ;
paramSensor.IMU.a13 = IMUXtalk13;
paramSensor.IMU.a32 = IMUXtalk32;
paramSensor.IMU.a12 = IMUXtalk12;
paramSensor.IMU.a1  = IMUOffsetX;
paramSensor.IMU.a2  = IMUOffsetY;
paramSensor.IMU.a3  = IMUOffsetZ;
paramSensor.IMU.sigmaNoise = IMUNoise;

%% run model
[t,x] = dynamic_model(initialConds, paramSC);

s = attitude_determination(t,x,paramSensor);
r = balancing_algorithm(t,s,paramSC);

% animation
if exist('visFlag','var') && visFlag
    visualize_motion(t,x,paramSC,'model_patch')
end

rg_sys = r;

